{***************************************************
  Freeware
  --------------------------------------------------
  Copyright  Scorpion 1997
  email: scorpion@sunpp.atom.gov.ua
  --------------------------------------------------
  Extended label (as standard TLabel & more)
  New features:
    3D drawing (Light insert & raised,
                heavy insert & raised)
    Drawing with shadow
    OnMouseEnter & OnMouseLeave events
  Compatible: D1(?), D2 & D3
***************************************************}

unit ExtLabel;
{$IFNDEF WIN32}
{$S-,W-,R-}
{$C PRELOAD}
{$ENDIF}

interface

uses WinTypes, WinProcs, Messages, SysUtils, Classes, Controls, Forms, Menus,
  Graphics;

type
  TDrawType = (dtNone,dtShadow,
               dtLightRaised,dtHeavyRaised,
               dtLightInsert,dtHeavyInsert);

  TCustomExtLabel = class(TGraphicControl)
  private
    FShadowIndent: Integer;
    FDrawType: TDrawType;
    FLightColor: TColor;
    FDarkColor: TColor;
    FFocusControl: TWinControl;
    FAlignment: TAlignment;
    FAutoSize: Boolean;
    FWordWrap: Boolean;
    FShowAccelChar: Boolean;
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    procedure AdjustBounds;
    procedure DoDrawText(var Rect: TRect; Flags: Word);
    procedure SetAlignment(Value: TAlignment);
    procedure SetAutoSize(Value: Boolean);
    procedure SetShowAccelChar(Value: Boolean);
    procedure SetWordWrap(Value: Boolean);
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMMouseEnter(var Msg: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Msg: TMessage); message CM_MOUSELEAVE;
    procedure SetLightColor(Color: TColor);
    procedure SetDarkColor(Color: TColor);
    procedure SetDrawType(DrawType: TDrawType);
    procedure SetShadowIndent(ShadowIndent: Integer);
    function GetTransparent: Boolean;
    procedure SetTransparent(Value: Boolean);
  protected
    procedure DoMouseEnter; virtual;
    procedure DoMouseLeave; virtual;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure Paint; override;
    property Alignment: TAlignment read FAlignment write SetAlignment
             default taLeftJustify;
    property AutoSize: Boolean read FAutoSize write SetAutoSize default True;
    property DarkColor: TColor read FDarkColor write SetDarkColor;
    property DrawType: TDrawType read FDrawType write SetDrawType;
    property FocusControl: TWinControl read FFocusControl write FFocusControl;
    property LightColor: TColor read FLightColor write SetLightColor;
    property ShowAccelChar: Boolean read FShowAccelChar write SetShowAccelChar default True;
    property Transparent: Boolean read GetTransparent write SetTransparent default False;
    property ShadowIndent: Integer read FShadowIndent write SetShadowIndent
             default 2;
    property WordWrap: Boolean read FWordWrap write SetWordWrap default False;
    {-----------------------------------------}
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  public
    constructor Create(AOwner: TComponent); override;
    property Canvas;
  end;

  TExtLabel = class(TCustomExtLabel)
  published
    property Align;
    property Alignment;
    property AutoSize;
    property Caption;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FocusControl;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Visible;
    property Transparent;
    property WordWrap;
    property LightColor;
    property DarkColor;
    property DrawType;
    property ShadowIndent;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseEnter;
    property OnMouseLeave;
  end;

procedure Register;

implementation

uses Consts;

{ TCustomExtLabel }

constructor TCustomExtLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque];
  Width := 66;
  Height := 18;
  FAutoSize := True;
  FShowAccelChar := True;
  FLightColor:=clWhite;
  FDarkColor:=clBlack;
  FDrawType:=dtLightRaised;
  FShadowIndent:=2;
end;

procedure TCustomExtLabel.DoDrawText(var Rect: TRect; Flags: Word);
var
  Text: array[0..255] of Char;
begin
  GetTextBuf(Text, SizeOf(Text));
  if (Flags and DT_CALCRECT <> 0) and ((Text[0] = #0) or FShowAccelChar and
    (Text[0] = '&') and (Text[1] = #0)) then StrCopy(Text, ' ');
  if not FShowAccelChar then Flags := Flags or DT_NOPREFIX;
  Canvas.Font := Font;
  if not Enabled then Canvas.Font.Color := clGrayText
  else
  begin
    case DrawType of
      dtLightRaised:
        begin
          Rect.Top  := Rect.Top-1;
          Rect.Left := Rect.Left-1;
          Canvas.Font.Color := FLightColor;
          DrawText(Canvas.Handle, Text, StrLen(Text), Rect, Flags);
          Rect.Top  := Rect.Top+1;
          Rect.Left := Rect.Left+1;
          Canvas.Font.Color := Font.Color;
        end;
      dtHeavyRaised:
        begin
          Rect.Top  := Rect.Top-1;
          Rect.Left := Rect.Left-1;
          Canvas.Font.Color := FLightColor;
          DrawText(Canvas.Handle, Text, StrLen(Text), Rect, Flags);
          Rect.Top  := Rect.Top+2;
          Rect.Left := Rect.Left+2;
          Canvas.Font.Color := FDarkColor;
          DrawText(Canvas.Handle, Text, StrLen(Text), Rect, Flags);
          Rect.Top  := Rect.Top-1;
          Rect.Left := Rect.Left-1;
          Canvas.Font.Color := Font.Color;
        end;
      dtLightInsert:
        begin
          Rect.Top  := Rect.Top+1;
          Rect.Left := Rect.Left+1;
          Canvas.Font.Color := FLightColor;
          DrawText(Canvas.Handle, Text, StrLen(Text), Rect, Flags);
          Rect.Top  := Rect.Top-1;
          Rect.Left := Rect.Left-1;
          Canvas.Font.Color := Font.Color;
        end;
      dtHeavyInsert:
        begin
          Rect.Top  := Rect.Top+1;
          Rect.Left := Rect.Left+1;
          Canvas.Font.Color := FLightColor;
          DrawText(Canvas.Handle, Text, StrLen(Text), Rect, Flags);
          Rect.Top  := Rect.Top-2;
          Rect.Left := Rect.Left-2;
          Canvas.Font.Color := FDarkColor;
          DrawText(Canvas.Handle, Text, StrLen(Text), Rect, Flags);
          Rect.Top  := Rect.Top+1;
          Rect.Left := Rect.Left+1;
          Canvas.Font.Color := Font.Color;
        end;
      dtShadow:
        begin
          if FShadowIndent>0 then
          begin
            Rect.Top  := Rect.Top+ShadowIndent;
            Rect.Left := Rect.Left+ShadowIndent;
          end;
          Canvas.Font.Color := FDarkColor;
          DrawText(Canvas.Handle, Text, StrLen(Text), Rect, Flags);
          Rect.Top  := Rect.Top-ShadowIndent;
          Rect.Left := Rect.Left-ShadowIndent;
          Canvas.Font.Color := Font.Color;
        end;
    end;
  end;
  DrawText(Canvas.Handle, Text, StrLen(Text), Rect, Flags);
  if (Enabled) and (FDrawType<>dtNone)
               and (FDrawType<>dtShadow)then
  begin
    Rect.Top:=Rect.Top-1;
    Rect.Left:=Rect.Left-1;
  end else
  if (Enabled) and (FDrawType=dtShadow)
               and (FShadowIndent<0) then
  begin
    Rect.Top  := Rect.Top+ShadowIndent;
    Rect.Left := Rect.Left+ShadowIndent;
  end;
end;

procedure TCustomExtLabel.Paint;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Rect: TRect;
begin
  with Canvas do
  begin
    if not Transparent then
    begin
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(ClientRect);
    end;
    Brush.Style := bsClear;
    Rect := ClientRect;
    DoDrawText(Rect, (DT_EXPANDTABS or DT_WORDBREAK) or
      Alignments[FAlignment]);
  end;
end;

procedure TCustomExtLabel.AdjustBounds;
const
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
var
  DC: HDC;
  X: Integer;
  Rect: TRect;
  ix: Integer;
begin
  if not (csReading in ComponentState) and FAutoSize then
  begin
    Rect := ClientRect;
    DC := GetDC(0);
    Canvas.Handle := DC;
    DoDrawText(Rect, (DT_EXPANDTABS or DT_CALCRECT) or WordWraps[FWordWrap]);
    Canvas.Handle := 0;
    ReleaseDC(0, DC);
    X := Left;
    if FAlignment = taRightJustify then Inc(X, Width - Rect.Right);
    if FDrawType=dtShadow then
      ix:=Abs(ShadowIndent)
    else
      ix:=2;
    SetBounds(X, Top, Rect.Right+ix, Rect.Bottom+ix)
  end;
end;

procedure TCustomExtLabel.SetAlignment(Value: TAlignment);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    Invalidate;
  end;
end;

procedure TCustomExtLabel.SetAutoSize(Value: Boolean);
begin
  if FAutoSize <> Value then
  begin
    FAutoSize := Value;
    AdjustBounds;
  end;
end;

procedure TCustomExtLabel.SetShowAccelChar(Value: Boolean);
begin
  if FShowAccelChar <> Value then
  begin
    FShowAccelChar := Value;
    Invalidate;
  end;
end;

procedure TCustomExtLabel.SetWordWrap(Value: Boolean);
begin
  if FWordWrap <> Value then
  begin
    FWordWrap := Value;
    AdjustBounds;
  end;
end;

procedure TCustomExtLabel.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FFocusControl) then
    FFocusControl := nil;
end;

procedure TCustomExtLabel.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
  AdjustBounds;
end;

procedure TCustomExtLabel.CMFontChanged(var Message: TMessage);
begin
  inherited;
  AdjustBounds;
end;

procedure TCustomExtLabel.CMDialogChar(var Message: TCMDialogChar);
begin
  if (FFocusControl <> nil) and Enabled and ShowAccelChar and
    IsAccel(Message.CharCode, Caption) then
    with FFocusControl do
      if CanFocus then
      begin
        SetFocus;
        Message.Result := 1;
      end;
end;

procedure TCustomExtLabel.DoMouseEnter;
begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TCustomExtLabel.DoMouseLeave;
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

procedure TCustomExtLabel.CMMouseEnter(var Msg: TMessage);
begin
  DoMouseEnter;
end;

procedure TCustomExtLabel.CMMouseLeave(var Msg: TMessage);
begin
  DoMouseLeave;
end;

procedure TCustomExtLabel.SetLightColor(Color: TColor);
begin
  FLightColor:=Color;
  Invalidate;
end;

procedure TCustomExtLabel.SetDarkColor(Color: TColor);
begin
  FDarkColor:=Color;
  Invalidate;
end;

procedure TCustomExtLabel.SetDrawType(DrawType: TDrawType);
begin
  FDrawType:=DrawType;
  AdjustBounds;
  Invalidate;
end;

procedure TCustomExtLabel.SetShadowIndent(ShadowIndent: Integer);
begin
  if (ShadowIndent>=-50) and (ShadowIndent<=50) then
    FShadowIndent:=ShadowIndent;
  if FDrawType=dtShadow then
  begin
    AdjustBounds;
    Invalidate;
  end;
end;

function TCustomExtLabel.GetTransparent: Boolean;
begin
  Result := not (csOpaque in ControlStyle);
end;

procedure TCustomExtLabel.SetTransparent(Value: Boolean);
begin
  if Transparent <> Value then
  begin
    if Value then
      ControlStyle := ControlStyle - [csOpaque] else
      ControlStyle := ControlStyle + [csOpaque];
    Invalidate;
  end;
end;

procedure Register;
begin
  RegisterComponents('Samples',[TExtLabel])
end;

end.

